@@ -42,19 +42,24 @@
 //
 //-----------------------------------------------------------------------------
 
-#include <ImfHeader.h>
-#include <ImfFrameBuffer.h>
-#include <ImathBox.h>
-#include <ImfTileDescription.h>
+#include "ImfHeader.h"
+#include "ImfFrameBuffer.h"
+#include "ImathBox.h"
+#include "ImfTileDescription.h"
+#include "ImfThreading.h"
+#include "ImfGenericOutputFile.h"
+#include "ImfForward.h"
+#include "ImfNamespace.h"
+#include "ImfExport.h"
+
+
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_ENTER
 
-namespace Imf {
 
-class TiledInputFile;
-class InputFile;
 struct PreviewRgba;
 
 
-class TiledOutputFile
+class TiledOutputFile : public GenericOutputFile
 {
   public:
 
@@ -73,23 +78,30 @@ class TiledOutputFile
     // subsampling is not supported.
     //
     // Tiles can be written to the file in arbitrary order.  The line
-    // order attribute can be used to cause be sorted in the file.
-    // When the file is read later, reading the tiles in the same order
-    // as they are in the file tends to be significantly faster than
-    // reading the tiles in random order (see writeTile, below).
+    // order attribute can be used to cause the tiles to be sorted in
+    // the file.  When the file is read later, reading the tiles in the
+    // same order as they are in the file tends to be significantly
+    // faster than reading the tiles in random order (see writeTile,
+    // below).
     //-------------------------------------------------------------------
     
-    TiledOutputFile (const char fileName[], const Header &header);
+    IMF_EXPORT
+    TiledOutputFile (const char fileName[],
+		     const Header &header,
+                     int numThreads = globalThreadCount ());
 
 
     // ----------------------------------------------------------------
     // A constructor that attaches the new TiledOutputFile object to
     // a file that has already been opened.  Destroying TiledOutputFile
     // objects constructed with this constructor does not automatically
-    // closse the corresponding files.
+    // close the corresponding files.
     // ----------------------------------------------------------------
 
-    TiledOutputFile (OStream &os, const Header &header);
+    IMF_EXPORT
+    TiledOutputFile (OPENEXR_IMF_INTERNAL_NAMESPACE::OStream &os,
+		     const Header &header,
+                     int numThreads = globalThreadCount ());
 
 
     //-----------------------------------------------------
@@ -99,6 +111,7 @@ class TiledOutputFile
     // have been written results in an incomplete file.
     //-----------------------------------------------------
     
+    IMF_EXPORT
     virtual ~TiledOutputFile ();
 
 
@@ -106,6 +119,7 @@ class TiledOutputFile
     // Access to the file name
     //------------------------
     
+    IMF_EXPORT
     const char *	fileName () const;
 
 
@@ -113,6 +127,7 @@ class TiledOutputFile
     // Access to the file header
     //--------------------------
     
+    IMF_EXPORT
     const Header &	header () const;
 
 
@@ -127,6 +142,7 @@ class TiledOutputFile
     // after each call to writeTile().
     //-------------------------------------------------------
     
+    IMF_EXPORT
     void		setFrameBuffer (const FrameBuffer &frameBuffer);
 
 
@@ -134,6 +150,7 @@ class TiledOutputFile
     // Access to the current frame buffer
     //-----------------------------------
     
+    IMF_EXPORT
     const FrameBuffer &	frameBuffer () const;
 
 
@@ -147,9 +164,13 @@ class TiledOutputFile
     // fields of the file header's TileDescriptionAttribute.
     //---------------------------------------------------------
 
+    IMF_EXPORT
     unsigned int	tileXSize () const;
+    IMF_EXPORT
     unsigned int	tileYSize () const;
+    IMF_EXPORT
     LevelMode		levelMode () const;
+    IMF_EXPORT
     LevelRoundingMode	levelRoundingMode () const;
 
 
@@ -189,16 +210,20 @@ class TiledOutputFile
     //      return value is the same as for numXLevels()
     //
     //	if levelMode() == RIPMAP_LEVELS:
-    //      an Iex::LogicExc exception is thrown
+    //      an IEX_NAMESPACE::LogicExc exception is thrown
     //
     // isValidLevel(lx, ly) returns true if the file contains 
     // a level with level number (lx, ly), false if not.
     //
     //--------------------------------------------------------------------
 
+    IMF_EXPORT
     int			numLevels () const;
+    IMF_EXPORT
     int			numXLevels () const;
+    IMF_EXPORT
     int			numYLevels () const;
+    IMF_EXPORT
     bool		isValidLevel (int lx, int ly) const;
 
 
@@ -220,7 +245,9 @@ class TiledOutputFile
     //
     //---------------------------------------------------------
 
+    IMF_EXPORT
     int			levelWidth  (int lx) const;
+    IMF_EXPORT
     int			levelHeight (int ly) const;
 
 
@@ -244,7 +271,9 @@ class TiledOutputFile
     //
     //----------------------------------------------------------
 
+    IMF_EXPORT
     int			numXTiles (int lx = 0) const;
+    IMF_EXPORT
     int			numYTiles (int ly = 0) const;
 
 
@@ -268,8 +297,10 @@ class TiledOutputFile
     //
     //---------------------------------------------------------
 
-    Imath::Box2i	dataWindowForLevel (int l = 0) const;
-    Imath::Box2i	dataWindowForLevel (int lx, int ly) const;
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i	dataWindowForLevel (int l = 0) const;
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i	dataWindowForLevel (int lx, int ly) const;
 
 
     //-------------------------------------------------------------------
@@ -293,10 +324,12 @@ class TiledOutputFile
     //
     //-------------------------------------------------------------------
 
-    Imath::Box2i	dataWindowForTile (int dx, int dy,
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i	dataWindowForTile (int dx, int dy,
 					   int l = 0) const;
 
-    Imath::Box2i	dataWindowForTile (int dx, int dy,
+    IMF_EXPORT
+    IMATH_NAMESPACE::Box2i	dataWindowForTile (int dx, int dy,
 					   int lx, int ly) const;
 
     //------------------------------------------------------------------
@@ -316,6 +349,15 @@ class TiledOutputFile
     // used for ONE_LEVEL and MIPMAP_LEVEL files.  It calls
     // writeTile(dx, dy, level, level).
     //
+    // The two writeTiles(dx1, dx2, dy1, dy2, ...) functions allow
+    // writing multiple tiles at once.  If multi-threading is used
+    // multiple tiles are written concurrently.  The tile coordinates,
+    // dx1, dx2 and dy1, dy2, specify inclusive ranges of tile
+    // coordinates.  It is valid for dx1 < dx2 or dy1 < dy2; the
+    // tiles are always written in the order specified by the line
+    // order attribute.  Hence, it is not possible to specify an
+    // "invalid" or empty tile range.
+    //
     // Pixels that are outside the pixel coordinate range for the tile's
     // level, are never accessed by writeTile().
     //
@@ -362,8 +404,18 @@ class TiledOutputFile
     //
     //------------------------------------------------------------------
 
-    void		writeTile (int dx, int dy, int l = 0);
-    void		writeTile (int dx, int dy, int lx, int ly);
+    IMF_EXPORT
+    void		writeTile  (int dx, int dy, int l = 0);
+    IMF_EXPORT
+    void		writeTile  (int dx, int dy, int lx, int ly);
+
+    IMF_EXPORT
+    void		writeTiles (int dx1, int dx2, int dy1, int dy2,
+                                    int lx, int ly);
+
+    IMF_EXPORT
+    void		writeTiles (int dx1, int dx2, int dy1, int dy2,
+                                    int l = 0);
 
 
     //------------------------------------------------------------------
@@ -374,9 +426,12 @@ class TiledOutputFile
     // "lineOrder", "channels", and "tiles" attributes must be the same.
     //------------------------------------------------------------------
     
+    IMF_EXPORT
     void		copyPixels (TiledInputFile &in);
+    IMF_EXPORT
+    void        copyPixels (TiledInputPart &in);
+    
     
-
     //------------------------------------------------------------------
     // Shortcut to copy all pixels from an InputFile into this file,
     // without uncompressing and then recompressing the pixel data.
@@ -387,8 +442,12 @@ class TiledOutputFile
     // To use this function, the InputFile must be tiled.
     //------------------------------------------------------------------
     
+    IMF_EXPORT
     void		copyPixels (InputFile &in);
+    IMF_EXPORT
+    void        copyPixels (InputPart &in);
 
+    
 
     //--------------------------------------------------------------
     // Updating the preview image:
@@ -396,7 +455,7 @@ class TiledOutputFile
     // updatePreviewImage() supplies a new set of pixels for the
     // preview image attribute in the file's header.  If the header
     // does not contain a preview image, updatePreviewImage() throws
-    // an Iex::LogicExc.
+    // an IEX_NAMESPACE::LogicExc.
     //
     // Note: updatePreviewImage() is necessary because images are
     // often stored in a file incrementally, a few tiles at a time,
@@ -408,13 +467,42 @@ class TiledOutputFile
     //
     //--------------------------------------------------------------
 
+    IMF_EXPORT
     void		updatePreviewImage (const PreviewRgba newPixels[]);
 
 
+    //-------------------------------------------------------------
+    // Break a tile -- for testing and debugging only:
+    // 
+    // breakTile(dx,dy,lx,ly,p,n,c) introduces an error into the
+    // output file by writing n copies of character c, starting
+    // p bytes from the beginning of the tile with tile coordinates
+    // (dx, dy) and level number (lx, ly).
+    //
+    // Warning: Calling this function usually results in a broken
+    // image file.  The file or parts of it may not be readable,
+    // or the file may contain bad data.
+    //
+    //-------------------------------------------------------------
+
+    IMF_EXPORT
+    void		breakTile  (int dx, int dy,
+				    int lx, int ly,
+				    int offset,
+				    int length,
+				    char c);
     struct Data;
 
   private:
 
+    // ----------------------------------------------------------------
+    // A constructor attaches the OutputStreamMutex to the
+    // given one from MultiPartOutputFile. Set the previewPosition
+    // and lineOffsetsPosition which have been acquired from
+    // the constructor of MultiPartOutputFile as well.
+    // ----------------------------------------------------------------
+    TiledOutputFile (const OutputPartData* part);
+
     TiledOutputFile (const TiledOutputFile &);		    // not implemented
     TiledOutputFile & operator = (const TiledOutputFile &); // not implemented
 
@@ -427,9 +515,14 @@ class TiledOutputFile
 					     int lx, int ly) const;
 
     Data *		_data;
+
+    OutputStreamMutex*  _streamData;
+    bool                _deleteStream;
+
+    friend class MultiPartOutputFile;
 };
 
 
-} // namespace Imf
+OPENEXR_IMF_INTERNAL_NAMESPACE_HEADER_EXIT
 
 #endif
